# koompi-drivers.py
#
# Copyright 2023 Mirko Brombin
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# SPDX-License-Identifier: GPL-3.0-only

import os
import logging
import tempfile
import subprocess
import shutil

logger = logging.getLogger("KOOMPI::KOOMPIDriver")


class KOOMPIDriver:

    def __init__(self):
        self.__binary = "/usr/bin/mhwd"
        self.__bin_pacman = "/usr/bin/pacman"

        if bin_mhwd := shutil.which("mhwd"):
            self.__binary = bin_mhwd

    def get_devices(self) -> list:
        logger.info("Getting devices list")
        devices = []
        output = subprocess.run([self.__binary, "-li"], capture_output=True, text=True).stdout

        for line in output.splitlines():
            if "pci" in line:
                device = {}
                device["modalias"] = line.split(":")[0].strip()
                devices.append(device)
                driver_line = next(output)
                while "pci" not in driver_line:
                    if "Name" in driver_line:
                        device["vendor"] = driver_line.split(":")[1].strip()
                    elif "Driver" in driver_line:
                        driver = {}
                        driver_name = driver_line.split(":")[1].strip()
                        driver["name"] = driver_name
                        driver["installed"] = self.__check_installation(driver_name)
                        if driver["installed"]:
                            devices[-1]["drivers"] = [driver]
                        else:
                            devices[-1]["drivers"] = []
                    try:
                        driver_line = next(output)
                    except StopIteration:
                        break

        logger.debug("Devices list: %s", devices)

        return devices

    def __check_installation(self, driver: str) -> bool:
        output = subprocess.run([self.__bin_pacman, "-Qq", driver], capture_output=True, text=True).stdout
        return output.strip() == driver

    def can_install(self) -> bool:
        return not os.path.exists("/tmp/abroot-transactions.lock")

    def get_install_command(self, drivers: list) -> str:
        command = ["pkexec", "pacman", "-S", "--no-confirm", "dkms", "nvidia-prime"]

        for driver in drivers:
            if "nvidia" in driver:
                command.append("nvidia-dkms")
            command.append(driver)

        logger.info("Install command: %s", " ".join(command))

        return " ".join(command)

    def autoinstall(self) -> bool:
        if "FAKE" in os.environ:
            logging.info(f"executing cmd: pkexec {self.__binary} autoinstall")
            return True

        proc = subprocess.Popen(["pkexec", self.__binary, "autoinstall"], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        out, err = proc.communicate()
        return proc.returncode == 0
